<?php
// v0.56
// see class_c64reader.php for more info

define("ERROR_NO_ERROR", 0);
define("ERROR_INVALID_POINTER", 1);
define("ERROR_CYCLE_DETECTED", 2);
define("ERROR_DIR_OFF_TRACK", 4);
define("ERROR_NO_END_DATA", 8);
define("ERROR_D64_ERROR_INFO", 16);

class DiskImage
{
	var $imagefile = "";
	var $title = "";
	var $error = "";
	var $error_bytes = "";
	var $entries = array();
	var $bam_message = "";
  var $filename = "";

	var $numtracks = 0;
	var $errorstart = 0;
	var $track_sizes = array();
	var $track_offsets = array(0);
	var $dir_entries_track = 0;
	var $dir_entries_sector = 0;

	// data structures for the disk layout
	//								 1   2   3   4   5   6   7   8   9  10  
	var $D64_track_sizes = array(0, 21, 21, 21, 21, 21, 21, 21, 21, 21, 21, 
									21, 21, 21, 21, 21, 21, 21, 19, 19, 19, 
									19, 19, 19, 19, 18, 18, 18, 18, 18, 18, 
									17, 17, 17, 17, 17, 17, 17, 17, 17, 17);
	var $D71_track_sizes = array(0, 21, 21, 21, 21, 21, 21, 21, 21, 21, 21, 
									21, 21, 21, 21, 21, 21, 21, 19, 19, 19, 
									19, 19, 19, 19, 18, 18, 18, 18, 18, 18, 
									17, 17, 17, 17, 17, 21, 21, 21, 21, 21, 
									21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
									21, 21, 19, 19, 19, 19, 19, 19, 19, 18, 
									18, 18, 18, 18, 18, 17, 17, 17, 17, 17);
	var $D81_track_sizes = array(0, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 
									40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 
									40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 
									40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 
									40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 
									40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 
									40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 
									40, 40, 40, 40, 40, 40, 40, 40, 40, 40);

	function DiskImage(&$image, &$filename)
	{
		$this->imagefile = $image;
    $this->filename = $filename;

		$imagesize = strlen($image);
		if ($imagesize == 819200 || $imagesize == 822400)
		{
			$this->numtracks = 80;
			$this->track_sizes = $this->D81_track_sizes;
			$this->errorstart = ($imagesize == 819200) ? 0 : 819200;
		}
		else if ($imagesize == 349696 || $imagesize == 351062)
		{
			$this->numtracks = 70;
			$this->track_sizes = $this->D71_track_sizes;
			$this->errorstart = ($imagesize == 349696) ? 0 : 349696;
		}
		else if ($imagesize == 196608 || $imagesize == 197376)
		{
			$this->numtracks = 40;
			$this->track_sizes = $this->D64_track_sizes;
			$this->errorstart = ($imagesize == 196608) ? 0 : 196608;
		}
		else if ($imagesize == 174848 || $imagesize == 175531)
		{
			$this->numtracks = 35;
			$this->track_sizes = $this->D64_track_sizes;
			$this->errorstart = ($imagesize == 174848) ? 0 : 174848;
		}
		else
		{
			$this->error = "WARNING: Encountered image with invalid size!: ".$this->filename."<br>\n";ob_flush();flush();
			$this->numtracks = 35;
			$this->track_sizes = $this->D64_track_sizes;
			$this->errorstart = 0;
		}

		if ($this->numtracks == 80)
		{
			$this->title = substr($image, 399364, 23); // Track 40, sector 0, offset 4
			$this->dir_entries_track = 40;
			$this->dir_entries_sector = 3;
		}
		else
		{
			$this->title = substr($image, 91536, 23); // Track 18, sector 0, offset 0xa0
			$this->bam_message = substr($image, 91559, 89); // Track 18, sector 0, offset 0xa0
			$this->dir_entries_track = 18;
			$this->dir_entries_sector = 1;
		}

		for ($i = 0; $i < $this->numtracks; $i++)
		{
			$this->track_offsets[] = $this->track_offsets[$i] + $this->track_sizes[$i];
		}
	}

	function is_valid($t, $s)
	{
		if ($t < 1 || $t > $this->numtracks)
			return False;
		if ($s < 0 || $s >= $this->track_sizes[$t])
			return False;
		return True;
	}

	function get_ptr($t, $s)
	{
		if (!$this->is_valid($t, $s))
			die("Tried to read an invalid track and sector!! t $t s $s<br>");

		$start = ($this->track_offsets[$t] + $s) << 8;

		if (strlen($this->imagefile) < $start + 2)
		{
			echo "WARNING: Undersized image, tried to read past end of image: ".$this->filename."<br>\n";ob_flush();flush();
			return "\0\0";
		}
		else
		{
			return substr($this->imagefile, $start, 2);
		}
	}

	function get_sector($t, $s)
	{
		if (!$this->is_valid($t, $s))
			die("Tried to read an invalid track and sector!! t $t s $s");

		$start = ($this->track_offsets[$t] + $s) << 8;

		if (strlen($this->imagefile) < $start + 256)
		{
			echo "WARNING: Undersized image, tried to read past end of image: ".$this->filename."<br>\n";ob_flush();flush();
			return str_repeat("\0", 256);
		}
		else
		{
			return substr($this->imagefile, $start, 256);
		}
	}

	function get_error_byte($t, $s)
	{
		if (!$this->is_valid($t, $s))
			die("Tried to read an invalid track and sector!! t $t s $s");

		if ($this->errorstart == 0)
			return 1;
		else
			return ord($this->imagefile[$this->errorstart + $this->track_offsets[$t] + $s]);
	}

	function get_chain($t, $s, $full_last_sector = False, $directory = False)
	{
		$data = "";
		$chain = array();
		$error = ERROR_NO_ERROR;

		while (True)
		{
			if (!$this->is_valid($t, $s))
			{
				$error |= ERROR_INVALID_POINTER;
				break;
			}

			if (in_array($t . "_" . $s, $chain))
			{
				$error |= ERROR_CYCLE_DETECTED;
				break;
			}

      $error_byte = $this->get_error_byte($t, $s);
      $error_bytes .= $error_byte;
      
			if ($error_byte != 1)
				$error |= ERROR_D64_ERROR_INFO;
			
			if ($directory && $t != 18)
				$error |= ERROR_DIR_OFF_TRACK;

			$chain[] = $t . "_" . $s;

			$sectdata = $this->get_sector($t, $s);

			if ($sectdata[0] == "\0")
			{
				$length = ord($sectdata[1]) - 1;

				if ($length < 1)
				{
					$error |= ERROR_NO_END_DATA;
					$length = 0;
				}

				if ($directory)
					$data .= $sectdata;
				else
					if ($full_last_sector)
						$data .= substr($sectdata, 2, 254);
					else
						$data .= substr($sectdata, 2, $length);

				break;
			}

			if ($directory)
				$data .= $sectdata;
			else
				$data .= substr($sectdata, 2, 254);

			$t = ord($sectdata[0]);
			$s = ord($sectdata[1]);
		}

		$ret = array();
		$ret["data"] = $data;
		$ret["error"] = $error;
		$ret["chain"] = $chain;
    $ret["error_bytes"] = $error_bytes;

		return $ret;
	}

	function get_dir()
	{
		$ret = $this->get_chain($this->dir_entries_track, $this->dir_entries_sector, False, True);

		for ($i = 2; $i < strlen($ret["data"]); $i += 32)
		{
			$entry = substr($ret["data"], $i, 30);
			if ($entry == str_pad("", 30, "\0"))
			continue;

			$file = array();
			$file["type"] = ord($entry[0]);
			$file["track"] = ord($entry[1]);
			$file["sector"] = ord($entry[2]);
			$file["name"] = substr($entry, 3, 16);
			$file["size_dir"] = ord($entry[28]) + (ord($entry[29]) << 8);
			$file["extradata"] = bin2hex(substr($entry, 19, 9));

			$this->entries[] = $file;
		}
		return $this->entries;
	}
}
?>
