<?php
// v0.46
// see class_c64reader.php for more info

class T64Image
{
	var $imagefile = "";
	var $signature = "";
	var $version = 0;
	var $max_entries = 0;
	var $used_entries = 0;   // DO NOT TRUST THIS
	var $title = "";
	var $error = "";
	var $entries = array();

	function T64Image(&$image)
	{
		$this->imagefile = $image;
		if (strlen($image) < 64)
		{
			$this->error = "The T64 image is undersized!";
			return;
		}

		$this->signature = substr($image, 0, 32);
		$u = unpack("vversion/vmaxentries/vusedentries", substr($image, 32, 6));
    $this->version = $u["version"];
		$this->max_entries = $u["maxentries"];
		$this->used_entries = $u["usedentries"];
		$this->title = substr($image, 40, 24);
		
    if (strlen($image) < (64 + 32 * $this->max_entries))
		{
			$this->error = "The T64 image doesn't contain enough entries for max_entries!".$this->max_entries;
			return;
		}

		// We build an array of file offsets for use later.
		// Note that the end of the file is also put in as an offset.
		$offsets = array(strlen($image));
		for ($i = 0; $i < $this->max_entries; $i++)
		{
			$start = 64 + 32 * $i;
			$u = unpack("Ctype/Cdisktype/vstartaddr/vendaddr/vdummy/Voffset", substr($image, $start, 12));
			if ($u["type"] != 0)
			{
          $entry = array();
					$entry["type"]      = $u["type"];
					$entry["disktype"]  = $u["disktype"];
					$entry["startaddr"] = $u["startaddr"];
					$entry["endaddr"]   = $u["endaddr"];
					$entry["offset"]    = $u["offset"];
					$entry["memsize"]   = $entry["endaddr"] - $entry["startaddr"] + 2;
					$entry["filename"]  = substr($image, $start + 16, 16);
					$entry["error"]     = (($entry["offset"]+$entry["memsize"]-2) > strlen($image)) ? 'offset outside imagesize' : '';
					$this->error        = $entry["error"] ? 'error in at least one entry' : '';
					$offsets[]          = $entry["offset"];
					$this->entries[]    = $entry;
			}
		}

		// The end address of a file inside a T64 CAN NOT BE TRUSTED.
		// So instead we locate the start offset of the next file and use that as the end.
    $Foffsets = array_flip($offsets);   
		for ($i = 0; $i < count($this->entries); $i++)
		{
      $startaddr = $this->entries[$i]["startaddr"];
			$offset    = $this->entries[$i]["offset"]; // getting current offset
			$pos       = $Foffsets[$offset]+1;         // using the next offset as the end		
			$endoffset = ( (count($this->entries)>1) && (count($this->entries)>=$pos) ) ? $offsets[$pos] : strlen($image);			
      $this->entries[$i]["endoffset"] = $endoffset;
      $this->entries[$i]["filesize"] = (($endoffset - $offset + 2)>=0) ? ($endoffset - $offset + 2) : 0; 
			$datasize = $endoffset - $offset;
      $this->entries[$i]["filedata"] = ($datasize>0) ? pack("v", $startaddr) . substr($image, $offset, $datasize) : "";
		}
	}
}
?>
